import logging
from .log_context import get_log_context


class CustomFormatter(logging.Formatter):
    def format(self, record):
        context = get_log_context()
        if len(context) > 0:
            context_str = ' ' + ' | '.join(context) + ' | '
            # Add the context information to the log message
            record.msg = f"{context_str}{record.msg}"

        logger_names_max_length = max(len('root'), max([len(logger.name) for logger in logging.Logger.manager.loggerDict.values() if isinstance(logger, logging.Logger)]))

        record.name = record.name[:logger_names_max_length].ljust(logger_names_max_length)
        record.levelname = record.levelname[:8].ljust(8)

        return super().format(record)


import logging
import logging.config
from .custom_formatter import CustomFormatter

def configure_logging(format: str = "%(asctime)s - %(name)s - %(levelname)s - %(message)s", level: str = "INFO"):

    logging.config.dictConfig({
        "version": 1,
        "disable_existing_loggers": True,
        "formatters": {
            "custom": {
                "()": CustomFormatter,
                "format": format
            }
        },
        "handlers": {
            "console": {
                "formatter": "custom",
                "class": "logging.StreamHandler",
            }
        },
        "loggers": {
            "": {"handlers": ["console"], "level": level},
        },
    })


import threading
import uuid
from typing import List, Optional

local_storage = threading.local() # each thread will have its own private copy!

def push_log_context(contextString: str):
    if not hasattr(local_storage, 'log_context'):
        local_storage.log_context = []
    local_storage.log_context.append(contextString)

def get_log_context() -> List[str]:
    if not hasattr(local_storage, 'log_context'):
        return []
    return local_storage.log_context

def pop_log_context():
    if not hasattr(local_storage, 'log_context'):
        return
    local_storage.log_context.pop()



class LogContext:

    def __init__(self, context_string: Optional[str] = None) -> None:
        # by default, if no context is provided, generate a random string
        if context_string is None:
            self._context_string =  uuid.uuid4().hex.upper()[:8]
        else:
            self._context_string = context_string

    def __enter__(self):
        push_log_context(self._context_string)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pop_log_context()